@@ -13,12 +13,28 @@ msc.peaks.write.csv = function(X, fname)
 	write.table(X, file=fname, sep=",", row.names=FALSE)	
 }
 
-msc.peaks.read.csv = function(fname)
+msc.peaks.read.csv = function(fname, mzXML.record=FALSE)
 {
   X = read.csv(fname, as.is=TRUE) # read input file with peaks
   CNames = c("Spectrum.Tag", "Spectrum.", "Intensity", "Substance.Mass") # column names
   if (!all(CNames %in% colnames(X))) 
     stop("Input variable 'X' is not in proper format") 
+  if (mzXML.record) { # create mzXML record 
+    S =  unique(X$Spectrum.)
+    sha1  = digest(fname, alg="sha1", file=TRUE)
+    mzXML$parentFile = paste(mzXML$parentFile, 
+      "    <parentFile filename='file:///", fname, 
+      "' fileType='processedData' fileSha1='",sha1,"'/>\n", sep="")
+    for (i in 1:length(S)) {
+      j = S[i]
+      scanOrigin = paste("<scanOrigin parentFileID='",sha1,"' num='",j,"'/>", sep="");
+      mzXML$scan[[i]] = list(mass=NULL, peaks=NULL, num=j, parentNum=j, 
+        msLevel=1, header=NULL, maldi=NULL, scanOrigin=scanOrigin,
+        precursorMz=NULL, nameValue=NULL)
+    }
+    attr(X,"mzXML") = mzXML
+  }
+  
   return(X)
 }
 
